@@ -39,6 +39,7 @@
 #include "nsCOMPtr.h"
 #include "nsIServiceManager.h"
 #include "nsIFragmentContentSink.h"
+#include "nsIDTD.h"
 #include "nsIHTMLContentSink.h"
 #include "nsIParser.h"
 #include "nsIParserService.h"
@@ -49,7 +50,7 @@
 #include "nsIDOMComment.h"
 #include "nsIDOMHTMLFormElement.h"
 #include "nsIDOMDocumentFragment.h"
-#include "nsVoidArray.h"
+#include "nsTArray.h"
 #include "nsINameSpaceManager.h"
 #include "nsIDocument.h"
 #include "nsINodeInfo.h"
@@ -65,6 +66,17 @@
 #include "nsContentSink.h"
 #include "nsTHashtable.h"
 #include "nsCycleCollectionParticipant.h"
+#include "nsICSSLoader.h"
+#include "nsICSSParser.h"
+#include "nsCSSProperty.h"
+#include "nsCSSDeclaration.h"
+#include "nsICSSStyleRule.h"
+#include "nsUnicharInputStream.h"
+#include "nsCSSStyleSheet.h"
+#include "nsICSSRuleList.h"
+#include "nsCSSDeclaration.h"
+#include "nsCSSProperty.h"
+#include "nsIDOMCSSRule.h"
 
 //
 // XXX THIS IS TEMPORARY CODE
@@ -76,6 +88,9 @@
 class nsHTMLFragmentContentSink : public nsIFragmentContentSink,
                                   public nsIHTMLContentSink {
 public:
+  /**
+   * @param aAllContent Whether there is context information available for the fragment.
+   */
   nsHTMLFragmentContentSink(PRBool aAllContent = PR_FALSE);
   virtual ~nsHTMLFragmentContentSink();
 
@@ -88,8 +103,8 @@ public:
 
   // nsIContentSink
   NS_IMETHOD WillParse(void) { return NS_OK; }
-  NS_IMETHOD WillBuildModel(void);
-  NS_IMETHOD DidBuildModel(void);
+  NS_IMETHOD WillBuildModel(nsDTDMode aDTDMode);
+  NS_IMETHOD DidBuildModel(PRBool aTerminated);
   NS_IMETHOD WillInterrupt(void);
   NS_IMETHOD WillResume(void);
   NS_IMETHOD SetParser(nsIParser* aParser);
@@ -139,8 +154,6 @@ public:
   void ProcessBaseTag(nsIContent* aContent);
   void AddBaseTagInfo(nsIContent* aContent);
 
-  nsresult Init();
-
   PRPackedBool mAllContent;
   PRPackedBool mProcessing;
   PRPackedBool mSeenBody;
@@ -150,7 +163,7 @@ public:
   nsCOMPtr<nsIContent> mRoot;
   nsCOMPtr<nsIParser> mParser;
 
-  nsVoidArray* mContentStack;
+  nsTArray<nsIContent*>* mContentStack;
 
   PRUnichar* mText;
   PRInt32 mTextLength;
@@ -210,9 +223,9 @@ nsHTMLFragmentContentSink::~nsHTMLFragmentContentSink()
 
   if (nsnull != mContentStack) {
     // there shouldn't be anything here except in an error condition
-    PRInt32 indx = mContentStack->Count();
+    PRInt32 indx = mContentStack->Length();
     while (0 < indx--) {
-      nsIContent* content = (nsIContent*)mContentStack->ElementAt(indx);
+      nsIContent* content = mContentStack->ElementAt(indx);
       NS_RELEASE(content);
     }
     delete mContentStack;
@@ -260,7 +273,7 @@ NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN(nsHTMLFragmentContentSink)
 NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
 
 NS_IMETHODIMP
-nsHTMLFragmentContentSink::WillBuildModel(void)
+nsHTMLFragmentContentSink::WillBuildModel(nsDTDMode)
 {
   if (mRoot) {
     return NS_OK;
@@ -278,7 +291,7 @@ nsHTMLFragmentContentSink::WillBuildModel(void)
 }
 
 NS_IMETHODIMP
-nsHTMLFragmentContentSink::DidBuildModel(void)
+nsHTMLFragmentContentSink::DidBuildModel(PRBool aTerminated)
 {
   FlushText();
 
@@ -419,7 +432,9 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
       ToLowerCase(tmp);
 
       nsCOMPtr<nsIAtom> name = do_GetAtom(tmp);
-      nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull, kNameSpaceID_None);
+      nodeInfo = mNodeInfoManager->GetNodeInfo(name, 
+                                               nsnull, 
+                                               kNameSpaceID_XHTML);
       NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
     }
     else if (mNodeInfoCache[nodeType]) {
@@ -433,7 +448,9 @@ nsHTMLFragmentContentSink::OpenContainer(const nsIParserNode& aNode)
       nsIAtom *name = parserService->HTMLIdToAtomTag(nodeType);
       NS_ASSERTION(name, "This should not happen!");
 
-      nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull, kNameSpaceID_None);
+      nodeInfo = mNodeInfoManager->GetNodeInfo(name, 
+                                               nsnull, 
+                                               kNameSpaceID_XHTML);
       NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
 
       NS_ADDREF(mNodeInfoCache[nodeType] = nodeInfo);
@@ -521,7 +538,7 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
 
           nsCOMPtr<nsIAtom> name = do_GetAtom(tmp);
           nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull,
-                                                   kNameSpaceID_None);
+                                                   kNameSpaceID_XHTML);
           NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
         }
         else if (mNodeInfoCache[nodeType]) {
@@ -532,7 +549,7 @@ nsHTMLFragmentContentSink::AddLeaf(const nsIParserNode& aNode)
           NS_ASSERTION(name, "This should not happen!");
 
           nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull,
-                                                   kNameSpaceID_None);
+                                                   kNameSpaceID_XHTML);
           NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
           NS_ADDREF(mNodeInfoCache[nodeType] = nodeInfo);
         }
@@ -662,7 +679,7 @@ nsHTMLFragmentContentSink::DidBuildContent()
 {
   if (!mAllContent) {
     FlushText();
-    DidBuildModel(); // Release our ref to the parser now.
+    DidBuildModel(PR_FALSE); // Release our ref to the parser now.
     mProcessing = PR_FALSE;
   }
 
@@ -680,9 +697,9 @@ nsIContent*
 nsHTMLFragmentContentSink::GetCurrentContent()
 {
   if (nsnull != mContentStack) {
-    PRInt32 indx = mContentStack->Count() - 1;
+    PRInt32 indx = mContentStack->Length() - 1;
     if (indx >= 0)
-      return (nsIContent *)mContentStack->ElementAt(indx);
+      return mContentStack->ElementAt(indx);
   }
   return nsnull;
 }
@@ -691,11 +708,11 @@ PRInt32
 nsHTMLFragmentContentSink::PushContent(nsIContent *aContent)
 {
   if (nsnull == mContentStack) {
-    mContentStack = new nsVoidArray();
+    mContentStack = new nsTArray<nsIContent*>();
   }
 
-  mContentStack->AppendElement((void *)aContent);
-  return mContentStack->Count();
+  mContentStack->AppendElement(aContent);
+  return mContentStack->Length();
 }
 
 nsIContent*
@@ -703,9 +720,9 @@ nsHTMLFragmentContentSink::PopContent()
 {
   nsIContent* content = nsnull;
   if (nsnull != mContentStack) {
-    PRInt32 indx = mContentStack->Count() - 1;
+    PRInt32 indx = mContentStack->Length() - 1;
     if (indx >= 0) {
-      content = (nsIContent *)mContentStack->ElementAt(indx);
+      content = mContentStack->ElementAt(indx);
       mContentStack->RemoveElementAt(indx);
     }
   }
@@ -869,10 +886,11 @@ nsHTMLFragmentContentSink::AddAttributes(const nsIParserNode& aNode,
 // Find the whitelist of allowed elements and attributes in
 // nsContentSink.h We share it with nsHTMLParanoidFragmentSink
 
-class nsHTMLParanoidFragmentSink : public nsHTMLFragmentContentSink
+class nsHTMLParanoidFragmentSink : public nsHTMLFragmentContentSink,
+                                   public nsIParanoidFragmentContentSink
 {
 public:
-  nsHTMLParanoidFragmentSink();
+  nsHTMLParanoidFragmentSink(PRBool aAllContent = PR_FALSE);
 
   static nsresult Init();
   static void Cleanup();
@@ -888,14 +906,24 @@ public:
 
   nsresult AddAttributes(const nsIParserNode& aNode,
                          nsIContent* aContent);
+
+  // nsIParanoidFragmentContentSink
+  virtual void AllowStyles();
+  virtual void AllowComments();
+
 protected:
   nsresult NameFromType(const nsHTMLTag aTag,
                         nsIAtom **aResult);
 
   nsresult NameFromNode(const nsIParserNode& aNode,
                         nsIAtom **aResult);
+
+  void SanitizeStyleRule(nsICSSStyleRule *aRule, nsAutoString &aRuleText);
   
-  PRBool mSkip; // used when we descend into <style> or <script>
+  PRPackedBool mSkip; // used when we descend into <style> or <script>
+  PRPackedBool mProcessStyle; // used when style is explicitly white-listed
+  PRPackedBool mInStyle; // whether we're inside a style element
+  PRPackedBool mProcessComments; // used when comments are allowed
 
   // Use nsTHashTable as a hash set for our whitelists
   static nsTHashtable<nsISupportsHashKey>* sAllowedTags;
@@ -905,8 +933,9 @@ protected:
 nsTHashtable<nsISupportsHashKey>* nsHTMLParanoidFragmentSink::sAllowedTags;
 nsTHashtable<nsISupportsHashKey>* nsHTMLParanoidFragmentSink::sAllowedAttributes;
 
-nsHTMLParanoidFragmentSink::nsHTMLParanoidFragmentSink():
-  nsHTMLFragmentContentSink(PR_FALSE), mSkip(PR_FALSE)
+nsHTMLParanoidFragmentSink::nsHTMLParanoidFragmentSink(PRBool aAllContent):
+  nsHTMLFragmentContentSink(aAllContent), mSkip(PR_FALSE),
+  mProcessStyle(PR_FALSE), mInStyle(PR_FALSE), mProcessComments(PR_FALSE)
 {
 }
 
@@ -977,14 +1006,27 @@ NS_NewHTMLParanoidFragmentSink(nsIFragmentContentSink** aResult)
   return NS_OK;
 }
 
+nsresult
+NS_NewHTMLParanoidFragmentSink2(nsIFragmentContentSink** aResult)
+{
+  nsHTMLParanoidFragmentSink* it = new nsHTMLParanoidFragmentSink(PR_TRUE);
+  if (!it) {
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+  nsresult rv = nsHTMLParanoidFragmentSink::Init();
+  NS_ENSURE_SUCCESS(rv, rv);
+  NS_ADDREF(*aResult = it);
+  
+  return NS_OK;
+}
+
 void
 NS_HTMLParanoidFragmentSinkShutdown()
 {
   nsHTMLParanoidFragmentSink::Cleanup();
 }
 
-NS_IMPL_ISUPPORTS_INHERITED0(nsHTMLParanoidFragmentSink,
-                             nsHTMLFragmentContentSink)
+NS_IMPL_ISUPPORTS_INHERITED1(nsHTMLParanoidFragmentSink, nsHTMLFragmentContentSink, nsIParanoidFragmentContentSink)
 
 nsresult
 nsHTMLParanoidFragmentSink::NameFromType(const nsHTMLTag aTag,
@@ -1011,7 +1053,7 @@ nsHTMLParanoidFragmentSink::NameFromNode(const nsIParserNode& aNode,
     nsCOMPtr<nsINodeInfo> nodeInfo;
     rv =
       mNodeInfoManager->GetNodeInfo(aNode.GetText(), nsnull,
-                                    kNameSpaceID_None,
+                                    kNameSpaceID_XHTML,
                                     getter_AddRefs(nodeInfo));
     NS_ENSURE_SUCCESS(rv, rv);
     NS_IF_ADDREF(*aResult = nodeInfo->NameAtom());
@@ -1021,6 +1063,18 @@ nsHTMLParanoidFragmentSink::NameFromNode(const nsIParserNode& aNode,
   return rv;
 }
 
+void
+nsHTMLParanoidFragmentSink::AllowStyles()
+{
+  mProcessStyle = PR_TRUE;
+}
+
+void
+nsHTMLParanoidFragmentSink::AllowComments()
+{
+  mProcessComments = PR_TRUE;
+}
+
 // nsHTMLFragmentContentSink
 
 nsresult
@@ -1051,7 +1105,10 @@ nsHTMLParanoidFragmentSink::AddAttributes(const nsIParserNode& aNode,
 
     // not an allowed attribute
     if (!sAllowedAttributes || !sAllowedAttributes->GetEntry(keyAtom)) {
-      continue;
+      // unless it's style, and we're allowing it
+      if (!mProcessStyle || keyAtom != nsGkAtoms::style) {
+        continue;
+      }
     }
 
     // Get value and remove mandatory quotes
@@ -1060,7 +1117,10 @@ nsHTMLParanoidFragmentSink::AddAttributes(const nsIParserNode& aNode,
       nsContentUtils::TrimCharsInSet(kWhitespace, aNode.GetValueAt(i));
 
     // check the attributes we allow that contain URIs
-    if (IsAttrURI(keyAtom)) {
+    // special case src attributes for img tags, because they can't
+    // run any dangerous code.
+    if (IsAttrURI(keyAtom) &&
+        !(nodeType == eHTMLTag_img && keyAtom == nsGkAtoms::src)) {
       if (!baseURI) {
         baseURI = aContent->GetBaseURI();
       }
@@ -1080,7 +1140,31 @@ nsHTMLParanoidFragmentSink::AddAttributes(const nsIParserNode& aNode,
       continue;
     }
 
-    if (nodeType == eHTMLTag_a && keyAtom == nsGkAtoms::name) {
+    // Filter unsafe stuff from style attributes if they're allowed
+    if (mProcessStyle && keyAtom == nsGkAtoms::style) {
+      if (!baseURI) {
+        baseURI = aContent->GetBaseURI();
+      }
+      nsICSSLoader* cssLoader = mTargetDocument->CSSLoader();
+      nsCOMPtr<nsICSSParser> parser;
+      rv = cssLoader->GetParserFor(nsnull, getter_AddRefs(parser));
+      if (NS_SUCCEEDED(rv)) {
+        nsCOMPtr<nsICSSStyleRule> rule;
+        rv = parser->ParseStyleAttribute(aNode.GetValueAt(i),
+                                         mTargetDocument->GetDocumentURI(),
+                                         baseURI,
+                                         mTargetDocument->NodePrincipal(),
+                                         getter_AddRefs(rule));
+        if (NS_SUCCEEDED(rv)) {
+          nsAutoString cleanValue;
+          SanitizeStyleRule(rule, cleanValue);
+          aContent->SetAttr(kNameSpaceID_None, keyAtom, cleanValue, PR_FALSE);
+        } else {
+          // we couldn't sanitize the style attribute, ignore it
+          continue;
+        }
+      }
+    } else if (nodeType == eHTMLTag_a && keyAtom == nsGkAtoms::name) {
       NS_ConvertUTF16toUTF8 cname(v);
       NS_ConvertUTF8toUTF16 uv(nsUnescape(cname.BeginWriting()));
       // Add attribute to content
@@ -1109,9 +1193,10 @@ nsHTMLParanoidFragmentSink::OpenContainer(const nsIParserNode& aNode)
 {
   nsresult rv = NS_OK;
   
-  // bail if it's a script or style, or we're already inside one of those
+  // bail if it's a script or style (when we don't allow processing of stylesheets),
+  // or we're already inside one of those
   eHTMLTags type = (eHTMLTags)aNode.GetNodeType();
-  if (type == eHTMLTag_script || type == eHTMLTag_style) {
+  if (type == eHTMLTag_script || (!mProcessStyle && type == eHTMLTag_style)) {
     mSkip = PR_TRUE;
     return rv;
   }
@@ -1122,7 +1207,14 @@ nsHTMLParanoidFragmentSink::OpenContainer(const nsIParserNode& aNode)
 
   // not on whitelist
   if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
-    return NS_OK;
+    // unless it's style, and we're allowing it
+    if (!mProcessStyle || name != nsGkAtoms::style) {
+      return NS_OK;
+    }
+  }
+
+  if (type == eHTMLTag_style) {
+    mInStyle = PR_TRUE;
   }
 
   return nsHTMLFragmentContentSink::OpenContainer(aNode);
@@ -1133,6 +1225,10 @@ nsHTMLParanoidFragmentSink::CloseContainer(const nsHTMLTag aTag)
 {
   nsresult rv = NS_OK;
 
+  if (mIgnoreNextCloseHead && aTag == eHTMLTag_head) {
+    mIgnoreNextCloseHead = PR_FALSE;
+    return NS_OK;
+  }
   if (mSkip) {
     mSkip = PR_FALSE;
     return rv;
@@ -1144,12 +1240,128 @@ nsHTMLParanoidFragmentSink::CloseContainer(const nsHTMLTag aTag)
   
   // not on whitelist
   if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
-    return NS_OK;
+    // unless it's style, and we're allowing it
+    if (!mProcessStyle || name != nsGkAtoms::style) {
+      return NS_OK;
+    }
+  }
+
+  if (mInStyle && name == nsGkAtoms::style) {
+    mInStyle = PR_FALSE;
+
+    // Flush the text to make sure that the style text is complete.
+    FlushText();
+
+    // sanitizedStyleText will hold the permitted CSS text.
+    // We use a white-listing approach, so we explicitly allow
+    // the CSS style and font-face rule types.  We also clear
+    // -moz-binding CSS properties.
+    nsAutoString sanitizedStyleText;
+    nsIContent* style = GetCurrentContent();
+    if (style) {
+      // styleText will hold the text inside the style element.
+      nsAutoString styleText;
+      nsContentUtils::GetNodeTextContent(style, PR_FALSE, styleText);
+      // Create a unichar input stream for the CSS parser.
+      nsCOMPtr<nsIUnicharInputStream> uin;
+      rv = nsSimpleUnicharStreamFactory::GetInstance()->
+        CreateInstanceFromString(styleText, getter_AddRefs(uin));
+      if (NS_SUCCEEDED(rv)) {
+        // Create a sheet to hold the parsed CSS
+        nsRefPtr<nsICSSStyleSheet> sheet;
+        rv = NS_NewCSSStyleSheet(getter_AddRefs(sheet));
+        if (NS_SUCCEEDED(rv)) {
+          nsCOMPtr<nsIURI> baseURI = style->GetBaseURI();
+          sheet->SetURIs(mTargetDocument->GetDocumentURI(), nsnull, baseURI);
+          sheet->SetPrincipal(mTargetDocument->NodePrincipal());
+          // Create the CSS parser, and parse the CSS text.
+          nsICSSLoader* cssLoader = mTargetDocument->CSSLoader();
+          nsCOMPtr<nsICSSParser> parser;
+          rv = cssLoader->GetParserFor(sheet, getter_AddRefs(parser));
+          if (NS_SUCCEEDED(rv)) {
+            rv = parser->Parse(uin, mTargetDocument->GetDocumentURI(),
+                               baseURI, mTargetDocument->NodePrincipal(),
+                               0, PR_FALSE);
+          }
+          // Mark the sheet as complete.
+          if (NS_SUCCEEDED(rv)) {
+            sheet->SetModified(PR_FALSE);
+            sheet->SetComplete();
+          }
+          if (NS_SUCCEEDED(rv)) {
+            // Loop through all the rules found in the CSS text
+            PRInt32 ruleCount = 0;
+            sheet->StyleRuleCount(ruleCount);
+            for (PRInt32 i = 0; i < ruleCount; ++i) {
+              nsRefPtr<nsICSSRule> rule;
+              rv = sheet->GetStyleRuleAt(i, *getter_AddRefs(rule));
+              if (NS_FAILED(rv))
+                continue;
+              NS_ASSERTION(rule, "We should have a rule by now");
+              PRInt32 type;
+              rv = rule->GetType(type);
+              if (NS_FAILED(rv))
+                continue;
+              switch (type) {
+                case nsICSSRule::UNKNOWN_RULE:
+                case nsICSSRule::CHARSET_RULE:
+                case nsICSSRule::IMPORT_RULE:
+                case nsICSSRule::MEDIA_RULE:
+                case nsICSSRule::PAGE_RULE:
+                  // Ignore these rule types.
+                  break;
+                case nsICSSRule::NAMESPACE_RULE:
+                case nsICSSRule::FONT_FACE_RULE: {
+                  // Append @namespace and @font-face rules verbatim.
+                  nsAutoString cssText;
+                  nsCOMPtr<nsIDOMCSSRule> styleRule = do_QueryInterface(rule);
+                  if (styleRule) {
+                    rv = styleRule->GetCssText(cssText);
+                    if (NS_SUCCEEDED(rv)) {
+                      sanitizedStyleText.Append(cssText);
+                    }
+                  }
+                  break;
+                }
+                case nsICSSRule::STYLE_RULE: {
+                  // For style rules, we will just look for and remove the
+                  // -moz-binding properties.
+                  nsCOMPtr<nsICSSStyleRule> styleRule = do_QueryInterface(rule);
+                  NS_ASSERTION(styleRule, "Must be a style rule");
+                  nsAutoString decl;
+                  SanitizeStyleRule(styleRule, decl);
+                  rv = styleRule->GetCssText(decl);
+                  // Only add the rule when sanitized.
+                  if (NS_SUCCEEDED(rv)) {
+                    sanitizedStyleText.Append(decl);
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+      // Replace the style element content with its sanitized style text
+      nsContentUtils::SetNodeTextContent(style, sanitizedStyleText, PR_TRUE);
+    }
   }
 
   return nsHTMLFragmentContentSink::CloseContainer(aTag);
 }
 
+void
+nsHTMLParanoidFragmentSink::SanitizeStyleRule(nsICSSStyleRule *aRule, nsAutoString &aRuleText)
+{
+  aRuleText.Truncate();
+  nsCSSDeclaration *style = aRule->GetDeclaration();
+  if (style) {
+    nsresult rv = style->RemoveProperty(eCSSProperty_binding);
+    if (NS_SUCCEEDED(rv)) {
+      style->ToString(aRuleText);
+    }
+  }
+}
+
 NS_IMETHODIMP
 nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
 {
@@ -1157,7 +1369,10 @@ nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
   
   nsresult rv = NS_OK;
 
-  if (mSkip) {
+  // We need to explicitly skip adding leaf nodes in the paranoid sink,
+  // otherwise things like the textnode under <title> get appended to
+  // the fragment itself, and won't be popped off in CloseContainer.
+  if (mSkip || mIgnoreNextCloseHead) {
     return rv;
   }
   
@@ -1175,7 +1390,7 @@ nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
       if (!parserService)
         return NS_ERROR_OUT_OF_MEMORY;
       nodeInfo = mNodeInfoManager->GetNodeInfo(name, nsnull,
-                                               kNameSpaceID_None);
+                                               kNameSpaceID_XHTML);
       NS_ENSURE_TRUE(nodeInfo, NS_ERROR_OUT_OF_MEMORY);
       rv = NS_NewHTMLElement(getter_AddRefs(content), nodeInfo, PR_FALSE);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -1185,7 +1400,9 @@ nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
     }
 
     if (!sAllowedTags || !sAllowedTags->GetEntry(name)) {
-      return NS_OK;
+      if (!mProcessStyle || name != nsGkAtoms::style) {
+        return NS_OK;
+      }
     }
   }
 
@@ -1195,6 +1412,8 @@ nsHTMLParanoidFragmentSink::AddLeaf(const nsIParserNode& aNode)
 NS_IMETHODIMP
 nsHTMLParanoidFragmentSink::AddComment(const nsIParserNode& aNode)
 {
+  if (mProcessComments)
+    return nsHTMLFragmentContentSink::AddComment(aNode);
   // no comments
   return NS_OK;
 }
