@@ -457,9 +457,7 @@ PluginInstanceChild::AnswerNPP_SetValue_NPNVprivateModeBool(const bool& value,
         return true;
     }
 
-    // Use `long` instead of NPBool because Flash and other plugins read
-    // this as a word-size value instead of the 1-byte NPBool that it is.
-    long v = value;
+    NPBool v = value;
     *result = mPluginIface->setvalue(GetNPP(), NPNVprivateModeBool, &v);
     return true;
 }
@@ -835,7 +833,7 @@ PluginInstanceChild::PluginWindowProc(HWND hWnd,
 
     // The plugin received keyboard focus, let the parent know so the dom is up to date.
     if (message == WM_MOUSEACTIVATE)
-        self->CallPluginGotFocus();
+      self->CallPluginFocusChange(PR_TRUE);
 
     // Prevent lockups due to plugins making rpc calls when the parent
     // is making a synchronous SendMessage call to the child window. Add
@@ -852,6 +850,9 @@ PluginInstanceChild::PluginWindowProc(HWND hWnd,
         }
     }
 
+    if (message == WM_KILLFOCUS)
+      self->CallPluginFocusChange(PR_FALSE);
+
     if (message == WM_USER+1 &&
         (self->mQuirks & PluginInstanceChild::QUIRK_FLASH_THROTTLE_WMUSER_EVENTS)) {
         self->FlashThrottleMessage(hWnd, message, wParam, lParam, true);
