@@ -15,12 +15,13 @@
  * The Original Code is Mozilla Foundation code.
  *
  * The Initial Developer of the Original Code is Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Portions created by the Initial Developer are Copyright (C) 2005-2009
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
  *   Stuart Parmenter <stuart@mozilla.com>
  *   John Daggett <jdaggett@mozilla.com>
+ *   Jonathan Kew <jfkthame@gmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -53,6 +54,7 @@
 #include "nsComponentManagerUtils.h"
 #include "nsTArray.h"
 #include "nsAutoPtr.h"
+#include "nsIStreamBufferAccess.h"
 
 /* Bug 341128 - w32api defines min/max which causes problems with <bitset> */
 #ifdef __MINGW32__
@@ -60,10 +62,6 @@
 #undef max
 #endif
 
-#include <bitset>
-
-// code from gfxWindowsFonts.h
-
 class gfxSparseBitSet {
 private:
     enum { BLOCK_SIZE = 32 };   // ==> 256 codepoints per block
@@ -292,9 +290,160 @@ public:
 
 #define TRUETYPE_TAG(a, b, c, d) ((a) << 24 | (b) << 16 | (c) << 8 | (d))
 
+namespace mozilla {
+
+// Byte-swapping types and name table structure definitions moved from
+// gfxFontUtils.cpp to .h file so that gfxFont.cpp can also refer to them
+#pragma pack(1)
+
+struct AutoSwap_PRUint16 {
+#ifdef __SUNPRO_CC
+    AutoSwap_PRUint16& operator = (const PRUint16 aValue)
+      { this->value = NS_SWAP16(aValue); return *this; }
+#else
+    AutoSwap_PRUint16(PRUint16 aValue) { value = NS_SWAP16(aValue); }
+#endif
+    operator PRUint16() const { return NS_SWAP16(value); }
+    operator PRUint32() const { return NS_SWAP16(value); }
+    operator PRUint64() const { return NS_SWAP16(value); }
+    PRUint16 value;
+};
+
+struct AutoSwap_PRInt16 {
+#ifdef __SUNPRO_CC
+    AutoSwap_PRInt16& operator = (const PRInt16 aValue)
+      { this->value = NS_SWAP16(aValue); return *this; }
+#else
+    AutoSwap_PRInt16(PRInt16 aValue) { value = NS_SWAP16(aValue); }
+#endif
+    operator PRInt16() const { return NS_SWAP16(value); }
+    operator PRUint32() const { return NS_SWAP16(value); }
+    PRInt16  value;
+};
+
+struct AutoSwap_PRUint32 {
+#ifdef __SUNPRO_CC
+    AutoSwap_PRUint32& operator = (const PRUint32 aValue)
+      { this->value = NS_SWAP32(aValue); return *this; }
+#else
+    AutoSwap_PRUint32(PRUint32 aValue) { value = NS_SWAP32(aValue); }
+#endif
+    operator PRUint32() const { return NS_SWAP32(value); }
+    PRUint32  value;
+};
+
+struct AutoSwap_PRUint64 {
+#ifdef __SUNPRO_CC
+    AutoSwap_PRUint64& operator = (const PRUint64 aValue)
+      { this->value = NS_SWAP64(aValue); return *this; }
+#else
+    AutoSwap_PRUint64(PRUint64 aValue) { value = NS_SWAP64(aValue); }
+#endif
+    operator PRUint64() const { return NS_SWAP64(value); }
+    PRUint64  value;
+};
+
+#pragma pack()
+
+} // namespace mozilla
+
+// used for overlaying name changes without touching original font data
+struct FontDataOverlay {
+    // overlaySrc != 0 ==> use overlay
+    PRUint32  overlaySrc;    // src offset from start of font data
+    PRUint32  overlaySrcLen; // src length
+    PRUint32  overlayDest;   // dest offset from start of font data
+};
+    
+enum gfxUserFontType {
+    GFX_USERFONT_UNKNOWN = 0,
+    GFX_USERFONT_OPENTYPE = 1,
+    GFX_USERFONT_SVG = 2,
+    GFX_USERFONT_WOFF = 3
+};
+
 class THEBES_API gfxFontUtils {
 
 public:
+    // these are public because gfxFont.cpp also looks into the name table
+    enum {
+        NAME_ID_FAMILY = 1,
+        NAME_ID_STYLE = 2,
+        NAME_ID_UNIQUE = 3,
+        NAME_ID_FULL = 4,
+        NAME_ID_VERSION = 5,
+        NAME_ID_POSTSCRIPT = 6,
+        NAME_ID_PREFERRED_FAMILY = 16,
+        NAME_ID_PREFERRED_STYLE = 17,
+
+        PLATFORM_ALL = -1,
+        PLATFORM_ID_UNICODE = 0,           // Mac OS uses this typically
+        PLATFORM_ID_MAC = 1,
+        PLATFORM_ID_ISO = 2,
+        PLATFORM_ID_MICROSOFT = 3,
+
+        ENCODING_ID_MAC_ROMAN = 0,         // traditional Mac OS script manager encodings
+        ENCODING_ID_MAC_JAPANESE = 1,      // (there are others defined, but some were never
+        ENCODING_ID_MAC_TRAD_CHINESE = 2,  // implemented by Apple, and I have never seen them
+        ENCODING_ID_MAC_KOREAN = 3,        // used in font names)
+        ENCODING_ID_MAC_ARABIC = 4,
+        ENCODING_ID_MAC_HEBREW = 5,
+        ENCODING_ID_MAC_GREEK = 6,
+        ENCODING_ID_MAC_CYRILLIC = 7,
+        ENCODING_ID_MAC_DEVANAGARI = 9,
+        ENCODING_ID_MAC_GURMUKHI = 10,
+        ENCODING_ID_MAC_GUJARATI = 11,
+        ENCODING_ID_MAC_SIMP_CHINESE = 25,
+
+        ENCODING_ID_MICROSOFT_SYMBOL = 0,  // Microsoft platform encoding IDs
+        ENCODING_ID_MICROSOFT_UNICODEBMP = 1,
+        ENCODING_ID_MICROSOFT_SHIFTJIS = 2,
+        ENCODING_ID_MICROSOFT_PRC = 3,
+        ENCODING_ID_MICROSOFT_BIG5 = 4,
+        ENCODING_ID_MICROSOFT_WANSUNG = 5,
+        ENCODING_ID_MICROSOFT_JOHAB  = 6,
+        ENCODING_ID_MICROSOFT_UNICODEFULL = 10,
+
+        LANG_ALL = -1,
+        LANG_ID_MAC_ENGLISH = 0,      // many others are defined, but most don't affect
+        LANG_ID_MAC_HEBREW = 10,      // the charset; should check all the central/eastern
+        LANG_ID_MAC_JAPANESE = 11,    // european codes, though
+        LANG_ID_MAC_ARABIC = 12,
+        LANG_ID_MAC_ICELANDIC = 15,
+        LANG_ID_MAC_TURKISH = 17,
+        LANG_ID_MAC_TRAD_CHINESE = 19,
+        LANG_ID_MAC_URDU = 20,
+        LANG_ID_MAC_KOREAN = 23,
+        LANG_ID_MAC_POLISH = 25,
+        LANG_ID_MAC_FARSI = 31,
+        LANG_ID_MAC_SIMP_CHINESE = 33,
+        LANG_ID_MAC_ROMANIAN = 37,
+        LANG_ID_MAC_CZECH = 38,
+        LANG_ID_MAC_SLOVAK = 39,
+
+        LANG_ID_MICROSOFT_EN_US = 0x0409,        // with Microsoft platformID, EN US lang code
+        
+        CMAP_MAX_CODEPOINT = 0x10ffff     // maximum possible Unicode codepoint 
+                                          // contained in a cmap
+    };
+
+    // name table has a header, followed by name records, followed by string data
+    struct NameHeader {
+        mozilla::AutoSwap_PRUint16    format;       // Format selector (=0).
+        mozilla::AutoSwap_PRUint16    count;        // Number of name records.
+        mozilla::AutoSwap_PRUint16    stringOffset; // Offset to start of string storage
+                                                    // (from start of table)
+    };
+
+    struct NameRecord {
+        mozilla::AutoSwap_PRUint16    platformID;   // Platform ID
+        mozilla::AutoSwap_PRUint16    encodingID;   // Platform-specific encoding ID
+        mozilla::AutoSwap_PRUint16    languageID;   // Language ID
+        mozilla::AutoSwap_PRUint16    nameID;       // Name ID.
+        mozilla::AutoSwap_PRUint16    length;       // String length (in bytes).
+        mozilla::AutoSwap_PRUint16    offset;       // String offset from start of storage
+                                                    // (in bytes).
+    };
 
     // for reading big-endian font data on either big or little-endian platforms
     
@@ -307,7 +456,7 @@ public:
     static inline PRUint16
     ReadShortAt16(const PRUint16 *aBuf, PRUint32 aIndex)
     {
-        const PRUint8 *buf = (PRUint8*) aBuf;
+        const PRUint8 *buf = reinterpret_cast<const PRUint8*>(aBuf);
         PRUint32 index = aIndex << 1;
         return (buf[index] << 8) | buf[index+1];
     }
@@ -327,11 +476,22 @@ public:
     ReadCMAPTableFormat4(PRUint8 *aBuf, PRUint32 aLength, 
                          gfxSparseBitSet& aCharacterMap);
 
+    static PRUint32
+    FindPreferredSubtable(PRUint8 *aBuf, PRUint32 aBufLength,
+                          PRUint32 *aTableOffset, PRBool *aSymbolEncoding);
+
     static nsresult
     ReadCMAP(PRUint8 *aBuf, PRUint32 aBufLength, gfxSparseBitSet& aCharacterMap,
              PRPackedBool& aUnicodeFont, PRPackedBool& aSymbolFont);
 
+    static PRUint32
+    MapCharToGlyphFormat4(const PRUint8 *aBuf, PRUnichar aCh);
+
+    static PRUint32
+    MapCharToGlyph(PRUint8 *aBuf, PRUint32 aBufLength, PRUnichar aCh);
+
 #ifdef XP_WIN
+
     // given a TrueType/OpenType data file, produce a EOT-format header
     // for use with Windows T2Embed API AddFontResource type API's
     // effectively hide existing fonts with matching names aHeaderLen is
@@ -339,14 +499,23 @@ public:
     // EOT header on output
     static nsresult
     MakeEOTHeader(const PRUint8 *aFontData, PRUint32 aFontDataLength,
-                  nsTArray<PRUint8> *aHeader);
+                  nsTArray<PRUint8> *aHeader, FontDataOverlay *aOverlay);
+
+    // determine whether a font (which has already passed ValidateSFNTHeaders)
+    // is CFF format rather than TrueType
+    static PRBool
+    IsCffFont(const PRUint8* aFontData);
+
 #endif
 
+    // determine the format of font data
+    static gfxUserFontType
+    DetermineFontDataType(const PRUint8 *aFontData, PRUint32 aFontDataLength);
+
     // checks for valid SFNT table structure, returns true if valid
     // does *not* guarantee that all font data is valid
     static PRBool
-    ValidateSFNTHeaders(const PRUint8 *aFontData, PRUint32 aFontDataLength,
-                        PRBool *aIsCFF = nsnull);
+    ValidateSFNTHeaders(const PRUint8 *aFontData, PRUint32 aFontDataLength);
     
     // create a new name table and build a new font with that name table
     // appended on the end, returns true on success
@@ -354,25 +523,6 @@ public:
     RenameFont(const nsAString& aName, const PRUint8 *aFontData, 
                PRUint32 aFontDataLength, nsTArray<PRUint8> *aNewFont);
     
-    // constansts used with name table read methods
-    enum {
-        PLATFORM_ALL = -1,
-        PLATFORM_UNICODE = 0,
-        PLATFORM_MACINTOSH = 1,
-        PLATFORM_MICROSOFT = 3,
-        
-        NAME_LANG_ALL = -1,
-        
-        // name record id's
-        NAME_ID_FAMILY = 1,
-        NAME_ID_STYLE = 2,
-        NAME_ID_UNIQUE = 3,
-        NAME_ID_FULL = 4,  // used as key to GDI CreateFontIndirect
-        NAME_ID_VERSION = 5,
-        NAME_ID_POSTSCRIPT = 6,
-        NAME_ID_PREFERRED_FAMILY = 16       
-    };
-
     // read all names matching aNameID, returning in aNames array
     static nsresult
     ReadNames(nsTArray<PRUint8>& aNameTable, PRUint32 aNameID, 
@@ -384,10 +534,16 @@ public:
     ReadCanonicalName(nsTArray<PRUint8>& aNameTable, PRUint32 aNameID, 
                       nsString& aName);
       
-    static inline bool IsJoiner(PRUint32 ch) {
-        return (ch == 0x200C ||
-                ch == 0x200D ||
-                ch == 0x2060);
+    // convert a name from the raw name table data into an nsString,
+    // provided we know how; return PR_TRUE if successful, or PR_FALSE
+    // if we can't handle the encoding
+    static PRBool
+    DecodeFontName(const PRUint8 *aBuf, PRInt32 aLength, 
+                   PRUint32 aPlatformCode, PRUint32 aScriptCode,
+                   PRUint32 aLangCode, nsAString& dest);
+
+    static inline bool IsJoinCauser(PRUint32 ch) {
+        return (ch == 0x200D);
     }
 
     static inline bool IsInvalid(PRUint32 ch) {
@@ -408,6 +564,24 @@ protected:
     ReadNames(nsTArray<PRUint8>& aNameTable, PRUint32 aNameID, 
               PRInt32 aLangID, PRInt32 aPlatformID, nsTArray<nsString>& aNames);
 
+    // convert opentype name-table platform/encoding/language values to a charset name
+    // we can use to convert the name data to unicode, or "" if data is UTF16BE
+    static const char*
+    GetCharsetForFontName(PRUint16 aPlatform, PRUint16 aScript, PRUint16 aLanguage);
+
+    struct MacFontNameCharsetMapping {
+        PRUint16    mEncoding;
+        PRUint16    mLanguage;
+        const char *mCharsetName;
+
+        bool operator<(const MacFontNameCharsetMapping& rhs) const {
+            return (mEncoding < rhs.mEncoding) ||
+                   ((mEncoding == rhs.mEncoding) && (mLanguage < rhs.mLanguage));
+        }
+    };
+    static const MacFontNameCharsetMapping gMacFontNameCharsets[];
+    static const char* gISOFontNameCharsets[];
+    static const char* gMSFontNameCharsets[];
 };
 
 // helper class for loading in font info spaced out at regular intervals
@@ -495,7 +669,7 @@ protected:
     virtual void FinishLoader() = 0;
 
     static void LoaderTimerCallback(nsITimer *aTimer, void *aThis) {
-        gfxFontInfoLoader *loader = (gfxFontInfoLoader*) aThis;
+        gfxFontInfoLoader *loader = static_cast<gfxFontInfoLoader*>(aThis);
         loader->LoaderTimerFire();
     }
 
