@@ -40,9 +40,9 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '9.1.1.11.js';
+gTestfile = '9.2.1.10.js';
 
-START("9.1.1.11 - XML [[Insert]]");
+START("9.2.1.10 XMLList [[ResolveValue]]");
 
 var actual = 'Test not implemented';
 var expect = 'PASS';
@@ -50,3 +50,4 @@ var expect = 'PASS';
 TEST(1, expect, actual);
 
 END();
+
