@@ -1,6 +1,5 @@
-/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- *
- * ***** BEGIN LICENSE BLOCK *****
+/* -*- Mode: java; tab-width:8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
  * The contents of this file are subject to the Mozilla Public License Version
@@ -13,18 +12,16 @@
  * for the specific language governing rights and limitations under the
  * License.
  *
- * The Original Code is Rhino code, released
- * May 6, 1999.
+ * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 1997-2000
+ * David P. Caldwell.
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s):
- *   Igor Bukanov
- *   Ethan Hugg
- *   Milen Nankov
+ *   Martin Honnen
+ *   David P. Caldwell <inonit@inonit.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -40,54 +37,16 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.4.4.23.js';
+gTestfile = '13.4.3.10.js';
 
-START("13.4.4.23 - XML namespace()");
+START("13.4.3.10 - XML Constructor [[HasInstance]]");
 
-TEST(1, true, XML.prototype.hasOwnProperty("namespace"));
+printBugNumber(288027);
 
-// Prefix case
-x =
-<foo:alpha xmlns:foo="http://foo/" xmlns:bar="http://bar/">
-    <foo:bravo>one</foo:bravo>
-</foo:alpha>;
+var xmlListObject1 = new XMLList('<god>Kibo</god>');
+var xmlListObject2 = new XMLList('<god>Kibo</god><devil>Xibo</devil>');
 
-y = x.namespace();
-TEST(2, "object", typeof(y));
-TEST(3, Namespace("http://foo/"), y);
+TEST(1, true, xmlListObject1 instanceof XML);
+TEST(2, true, xmlListObject2 instanceof XML);
 
-y = x.namespace("bar");
-TEST(4, "object", typeof(y));
-TEST(5, Namespace("http://bar/"), y);
-
-// No Prefix Case
-x =
-<alpha xmlns="http://foobar/">
-    <bravo>one</bravo>
-</alpha>;
-
-y = x.namespace();
-TEST(6, "object", typeof(y));
-TEST(7, Namespace("http://foobar/"), y);
-
-// No Namespace case
-x =
-<alpha>
-    <bravo>one</bravo>
-</alpha>;
-TEST(8, Namespace(""), x.namespace());
-
-// Namespaces of attributes
-x =
-<alpha xmlns="http://foo/">
-    <ns:bravo xmlns:ns="http://bar" name="two" ns:value="three">one</ns:bravo>
-</alpha>;
-
-var ns = new Namespace("http://bar");
-y = x.ns::bravo.@name.namespace();
-TEST(9, Namespace(""), y);
-
-y = x.ns::bravo.@ns::value.namespace();
-TEST(10, ns.toString(), y.toString());
-
-END();
\ No newline at end of file
+END();
