@@ -36,42 +36,50 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.9.5.12-5.js';
+gTestfile = '15.9.2.2-6.js';
 
 /**
-   File Name:          15.9.5.12.js
-   ECMA Section:       15.9.5.12
-   Description:        Date.prototype.getDay
-
-
-   1.   Let t be this time value.
-   2.   If t is NaN, return NaN.
-   3.   Return WeekDay(LocalTime(t)).
+   File Name:          15.9.2.2.js
+   ECMA Section:       15.9.2.2 Date constructor used as a function
+   Date( year, month, date, hours, minutes, seconds )
+   Description:        The arguments are accepted, but are completely ignored.
+   A string is created and returned as if by the
+   expression (new Date()).toString().
 
    Author:             christine@netscape.com
-   Date:               12 november 1997
+   Date:               28 october 1997
+   Version:            9706
+
 */
-
-var SECTION = "15.9.5.12";
-var VERSION = "ECMA_1";
+var VERSION = 9706;
 startTest();
-var TITLE   = "Date.prototype.getDay()";
+var SECTION = "15.9.2.2";
+var TOLERANCE = 100;
+var TITLE = "The Date Constructor Called as a Function";
 
-writeHeaderToLog( SECTION + " "+ TITLE);
+writeHeaderToLog(SECTION+" "+TITLE );
 
-addTestCase( TIME_2000 );
+// allow up to 1 second difference due to possibility
+// the date may change by 1 second in between calls to Date
+
+var d1;
+var d2;
+
+// Dates around jan 1, 2032
+d1 = new Date();
+d2 = Date.parse(Date(2031,11,31,23,59,59));
+new TestCase(SECTION, "Date(2031,11,31,23,59,59)", true, d2 - d1 <= 1000);
+
+d1 = new Date();
+d2 = Date.parse(Date(2032,0,1,0,0,0));
+new TestCase(SECTION, "Date(2032,0,1,0,0,0)", true, d2 - d1 <= 1000);
+
+d1 = new Date();
+d2 = Date.parse(Date(2032,0,1,0,0,1));
+new TestCase(SECTION, "Date(2032,0,1,0,0,1)", true, d2 - d1 <= 1000);
+
+d1 = new Date();
+d2 = Date.parse(Date(2031,11,31,16,0,0,0));
+new TestCase(SECTION, "Date(2031,11,31,16,0,0,0)", true, d2 - d1 <= 1000);
 
 test();
-
-function addTestCase( t ) {
-  var start = TimeFromYear(YearFromTime(t));
-  var stop  = TimeFromYear(YearFromTime(t) + 1);
-
-  for (var d = start; d < stop; d += msPerDay)
-  {
-    new TestCase( SECTION,
-                  "(new Date("+d+")).getDay()",
-                  WeekDay((LocalTime(d))),
-                  (new Date(d)).getDay() );
-  }
-}
