@@ -16,7 +16,7 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2008
+ * Portions created by the Initial Developer are Copyright (C) 2007
  * the Initial Developer. All Rights Reserved.
  *
  * Contributor(s): Jesse Ruderman
@@ -35,14 +35,13 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-452498-187.js';
+var gTestfile = 'regress-373827-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
+var BUGNUMBER = 373827;
+var summary = 'Do not assert: OBJ_GET_CLASS(cx, obj)->flags & JSCLASS_HAS_PRIVATE';
 var actual = '';
 var expect = '';
 
-//-------  Comment #187  From  Jesse Ruderman
 
 //-----------------------------------------------------------------------------
 test();
@@ -53,16 +52,8 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
-
-  expect = 'SyntaxError: invalid for/in left-hand side';
-  try
-  {
-    eval('const x; for (x in []);');
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
+ 
+  let ([] = [{x: function(){}}]) { };
 
   reportCompare(expect, actual, summary);
 
