@@ -74,7 +74,6 @@ mailing address.
 #include <stddef.h>
 #include "prmem.h"
 
-#include "nsIImage.h"
 #include "nsIInterfaceRequestorUtils.h"
 
 #include "nsGIFDecoder2.h"
@@ -118,6 +117,7 @@ nsGIFDecoder2::nsGIFDecoder2()
   , mLastFlushedRow(-1)
   , mImageData(nsnull)
   , mOldColor(0)
+  , mCurrentFrame(-1)
   , mCurrentPass(0)
   , mLastFlushedPass(0)
   , mGIFOpen(PR_FALSE)
@@ -142,7 +142,7 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
 {
   mObserver = do_QueryInterface(aLoad);
 
-  mImageContainer = do_CreateInstance("@mozilla.org/image/container;1");
+  mImageContainer = do_CreateInstance("@mozilla.org/image/container;2");
   aLoad->SetImage(mImageContainer);
   
   // Start with the version (GIF89a|GIF87a)
@@ -161,7 +161,7 @@ NS_IMETHODIMP nsGIFDecoder2::Init(imgILoad *aLoad)
 /* void close (); */
 NS_IMETHODIMP nsGIFDecoder2::Close()
 {
-  if (mImageFrame) 
+  if (mCurrentFrame == mGIFStruct.images_decoded)
     EndImageFrame();
   EndGIF();
 
@@ -202,11 +202,10 @@ static NS_METHOD ReadDataOut(nsIInputStream* in,
 nsresult
 nsGIFDecoder2::FlushImageData(PRUint32 fromRow, PRUint32 rows)
 {
-  nsIntRect r(0, fromRow, mGIFStruct.screen_width, rows);
+  nsIntRect r(0, fromRow, mGIFStruct.width, rows);
 
   // Update image  
-  nsCOMPtr<nsIImage> img(do_GetInterface(mImageFrame));
-  nsresult rv = img->ImageUpdated(nsnull, nsImageUpdateFlags_kBitsChanged, &r);
+  nsresult rv = mImageContainer->FrameUpdated(mGIFStruct.images_decoded, r);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -214,8 +213,10 @@ nsGIFDecoder2::FlushImageData(PRUint32 fromRow, PRUint32 rows)
   // Offset to the frame position
   // Only notify observer(s) for first frame
   if (!mGIFStruct.images_decoded && mObserver) {
+    PRUint32 imgCurFrame;
+    mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
     r.y += mGIFStruct.y_offset;
-    mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
+    mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
   }
   return NS_OK;
 }
@@ -251,7 +252,7 @@ nsresult nsGIFDecoder2::ProcessData(unsigned char *data, PRUint32 count, PRUint3
   NS_ENSURE_SUCCESS(rv, rv);
 
   // Flushing is only needed for first frame
-  if (!mGIFStruct.images_decoded && mImageFrame) {
+  if (!mGIFStruct.images_decoded) {
     rv = FlushImageData();
     mLastFlushedRow = mCurrentRow;
     mLastFlushedPass = mCurrentPass;
@@ -322,10 +323,8 @@ void nsGIFDecoder2::EndGIF()
 }
 
 //******************************************************************************
-void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
+nsresult nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
 {
-  mImageFrame = nsnull; // clear out our current frame reference
-
   if (!mGIFStruct.images_decoded) {
     // Send a onetime OnDataAvailable (Display Refresh) for the first frame
     // if it has a y-axis offset.  Otherwise, the area may never be refreshed
@@ -333,41 +332,47 @@ void nsGIFDecoder2::BeginImageFrame(gfx_depth aDepth)
     if (mGIFStruct.y_offset > 0) {
       PRInt32 imgWidth;
       mImageContainer->GetWidth(&imgWidth);
+      PRUint32 imgCurFrame;
+      mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
       nsIntRect r(0, 0, imgWidth, mGIFStruct.y_offset);
-      mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
+      mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
     }
   }
 
+  PRUint32 imageDataLength;
+  nsresult rv;
+  gfxASurface::gfxImageFormat format;
+  if (mGIFStruct.is_transparent)
+    format = gfxASurface::ImageFormatARGB32;
+  else
+    format = gfxASurface::ImageFormatRGB24;
+
   // Use correct format, RGB for first frame, PAL for following frames
   // and include transparency to allow for optimization of opaque images
-  gfx_format format;
   if (mGIFStruct.images_decoded) {
     // Image data is stored with original depth and palette
-    format = mGIFStruct.is_transparent ? gfxIFormats::PAL_A1 : gfxIFormats::PAL;
+    rv = mImageContainer->AppendPalettedFrame(mGIFStruct.x_offset, mGIFStruct.y_offset,
+                                              mGIFStruct.width, mGIFStruct.height,
+                                              format, aDepth, &mImageData, &imageDataLength,
+                                              &mColormap, &mColormapSize);
   } else {
     // Regardless of depth of input, image is decoded into 24bit RGB
-    format = mGIFStruct.is_transparent ? gfxIFormats::RGB_A1 : gfxIFormats::RGB;
-    aDepth = 24;
+    rv = mImageContainer->AppendFrame(mGIFStruct.x_offset, mGIFStruct.y_offset,
+                                      mGIFStruct.width, mGIFStruct.height,
+                                      format, &mImageData, &imageDataLength);
   }
 
-  // initialize the frame and append it to the container
-  mImageFrame = do_CreateInstance("@mozilla.org/gfx/image/frame;2");
-  if (!mImageFrame || NS_FAILED(mImageFrame->Init(
-        mGIFStruct.x_offset, mGIFStruct.y_offset, 
-        mGIFStruct.width, mGIFStruct.height, format, aDepth))) {
-    mImageFrame = 0;
-    return;
-  }
+  if (NS_FAILED(rv))
+    return rv;
 
-  mImageFrame->SetFrameDisposalMethod(mGIFStruct.disposal_method);
-  if (!mGIFStruct.images_decoded)
-    mImageContainer->AppendFrame(mImageFrame);
+  mImageContainer->SetFrameDisposalMethod(mGIFStruct.images_decoded,
+                                          mGIFStruct.disposal_method);
 
   if (mObserver)
-    mObserver->OnStartFrame(nsnull, mImageFrame);
+    mObserver->OnStartFrame(nsnull, mGIFStruct.images_decoded);
 
-  PRUint32 imageDataLength;
-  mImageFrame->GetImageData(&mImageData, &imageDataLength);
+  mCurrentFrame = mGIFStruct.images_decoded;
+  return NS_OK;
 }
 
 
@@ -384,20 +389,23 @@ void nsGIFDecoder2::EndImageFrame()
     // This will clear the remaining bits of the placeholder. (Bug 37589)
     const PRUint32 realFrameHeight = mGIFStruct.height + mGIFStruct.y_offset;
     if (realFrameHeight < mGIFStruct.screen_height) {
+      PRUint32 imgCurFrame;
+      mImageContainer->GetCurrentFrameIndex(&imgCurFrame);
       nsIntRect r(0, realFrameHeight, 
                   mGIFStruct.screen_width, 
 				  mGIFStruct.screen_height - realFrameHeight);
-      mObserver->OnDataAvailable(nsnull, mImageFrame, &r);
+      mObserver->OnDataAvailable(nsnull, imgCurFrame == PRUint32(mGIFStruct.images_decoded), &r);
     }
     // This transparency check is only valid for first frame
     if (mGIFStruct.is_transparent && !mSawTransparency) {
-      nsCOMPtr<nsIImage> img(do_GetInterface(mImageFrame));
-      img->SetHasNoAlpha();
+      mImageContainer->SetFrameHasNoAlpha(mGIFStruct.images_decoded);
     }
   }
   mCurrentRow = mLastFlushedRow = -1;
   mCurrentPass = mLastFlushedPass = 0;
 
+  PRUint32 curframe = mGIFStruct.images_decoded;
+
   // Only add frame if we have any rows at all
   if (mGIFStruct.rows_remaining != mGIFStruct.height) {
     if (mGIFStruct.rows_remaining && mGIFStruct.images_decoded) {
@@ -410,10 +418,8 @@ void nsGIFDecoder2::EndImageFrame()
     // image data, at least according to the spec, but we delay in setting the 
     // timeout for the image until here to help ensure that we have the whole 
     // image frame decoded before we go off and try to display another frame.
-    mImageFrame->SetTimeout(mGIFStruct.delay_time);
-    if (mGIFStruct.images_decoded)
-      mImageContainer->AppendFrame(mImageFrame);
-    mImageContainer->EndFrameDecode(mGIFStruct.images_decoded, mGIFStruct.delay_time);
+    mImageContainer->SetFrameTimeout(mGIFStruct.images_decoded, mGIFStruct.delay_time);
+    mImageContainer->EndFrameDecode(mGIFStruct.images_decoded);
   }
 
   // Unconditionally increment images_decoded, because we unconditionally
@@ -423,16 +429,15 @@ void nsGIFDecoder2::EndImageFrame()
   mGIFStruct.images_decoded++;
 
   if (mObserver)
-    mObserver->OnStopFrame(nsnull, mImageFrame);
-
-  // Release reference to this frame
-  mImageFrame = nsnull;
+    mObserver->OnStopFrame(nsnull, curframe);
 
   // Reset the transparent pixel
   if (mOldColor) {
     mColormap[mGIFStruct.tpixel] = mOldColor;
     mOldColor = 0;
   }
+
+  mCurrentFrame = -1;
 }
 
 
@@ -510,7 +515,7 @@ PRUint32 nsGIFDecoder2::OutputRow()
     if (drow_end > drow_start) {
       // irow is the current row filled
       for (int r = drow_start; r <= drow_end; r++) {
-        if (r != mGIFStruct.irow) {
+        if (r != int(mGIFStruct.irow)) {
           memcpy(mImageData + (r * bpr), rowp, bpr);
         }
       }
@@ -1071,8 +1076,8 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
       } 
       // Mask to limit the color values within the colormap
       mColorMask = 0xFF >> (8 - realDepth);
-      BeginImageFrame(realDepth);
-      if (!mImageData) {
+      nsresult rv = BeginImageFrame(realDepth);
+      if (NS_FAILED(rv) || !mImageData) {
         mGIFStruct.state = gif_error;
         break;
       }
@@ -1098,17 +1103,12 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
       if (q[8] & 0x80) /* has a local colormap? */
       {
         mGIFStruct.local_colormap_size = 1 << depth;
-        PRUint32 paletteSize;
-        if (mGIFStruct.images_decoded) {
-          // Copy directly into the palette of current frame,
-          // by pointing mColormap to that palette.
-          mImageFrame->GetPaletteData(&mColormap, &paletteSize);
-        } else {
+        if (!mGIFStruct.images_decoded) {
           // First frame has local colormap, allocate space for it
           // as the image frame doesn't have its own palette
-          paletteSize = sizeof(PRUint32) << realDepth;
+          mColormapSize = sizeof(PRUint32) << realDepth;
           if (!mGIFStruct.local_colormap) {
-            mGIFStruct.local_colormap = (PRUint32*)PR_MALLOC(paletteSize);
+            mGIFStruct.local_colormap = (PRUint32*)PR_MALLOC(mColormapSize);
             if (!mGIFStruct.local_colormap) {
               mGIFStruct.state = gif_oom;
               break;
@@ -1117,9 +1117,9 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
           mColormap = mGIFStruct.local_colormap;
         }
         const PRUint32 size = 3 << depth;
-        if (paletteSize > size) {
+        if (mColormapSize > size) {
           // Clear the notfilled part of the colormap
-          memset(((PRUint8*)mColormap) + size, 0, paletteSize - size);
+          memset(((PRUint8*)mColormap) + size, 0, mColormapSize - size);
         }
         if (len < size) {
           // Use 'hold' pattern to get the image colormap
@@ -1134,12 +1134,11 @@ nsresult nsGIFDecoder2::GifWrite(const PRUint8 *buf, PRUint32 len)
         break;
       } else {
         /* Switch back to the global palette */
-        mColormap = mGIFStruct.global_colormap;
         if (mGIFStruct.images_decoded) {
           // Copy global colormap into the palette of current frame
-          PRUint32 size;
-          mImageFrame->GetPaletteData(&mColormap, &size);
-          memcpy(mColormap, mGIFStruct.global_colormap, size);
+          memcpy(mColormap, mGIFStruct.global_colormap, mColormapSize);
+        } else {
+          mColormap = mGIFStruct.global_colormap;
         }
       }
       GETN(1, gif_lzw_start);
